/*

Name:
TCLDISPLAY.C

Description:
TCL display section of mikmod 

Peter Amstutz <amstpi@freenet.tlh.fl.us>

HISTORY
=======

v1.00 (19/01/97) - split off from display.c to utilize TCL/TK frontend
v1.01 (27/03/97) - added version verification, so the version of the
					tcl/tk script has to match up with the version
					that tclmikmod expects.  Also upper-ascii translation,
					so mods that have ibm upper-ascii characters are translated
					into something that vaguly resembles what it was supposed
					to look like, rather than the gobbledygook that you end
					up with otherwise...  translation table is stored in
					"asc8-to-7.txt" (that is, 8 bit to 7 bit ascii)
*/

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include "mikmod.h"

extern curmod cur_mod;
extern int quiet;
extern float speed_constant;   /* multiplier for tempo,initialised to 1 */
int readme;
int writeme;
pid_t pid;
extern UNIMOD *mf;

void init_display(void)
{
	int pipe1[2];
	int pipe2[2];
	FILE *tclfile;
	char c;
	char verline[40];

	if(quiet) return;

	pipe(pipe1);
	pipe(pipe2);

	pid=fork();

	if(pid==0)
	{
		dup2(pipe1[0],STDIN_FILENO);
		close(pipe1[0]);
		dup2(pipe2[1],STDOUT_FILENO);
		close(pipe2[1]);
		execlp("wish","wish",NULL);
		perror("Error in fork() call to start wish!!!");
		exit(1);
	}
	readme=pipe2[0];
	writeme=pipe1[1];
	if((tclfile=fopen("start.tcl","r"))==NULL)
	{
		perror("Error reading start.tcl file!");
		exit(1);
	}
	fgets(verline,40,tclfile);
	if(strcmp(verline,"#V1.11\n")!=0) 
	{
		perror("start.tcl version mismatch!");
		exit(1);
	} 
	while(!feof(tclfile))
	{
		c=fgetc(tclfile);
		write(writeme,&c,1);
	}
	c='\n';
	write(writeme,&c,1);
	usleep(500000);
}

void display_version(void)
{
	if(quiet) return;
/*	printf("%s",cur_mod.version);*/
}	

void display_driver(void)
{
	if(quiet) return;
	write(writeme,cur_mod.driver,strlen(cur_mod.driver));
}	

void display_file(void)
{
	if(quiet) return;
/*	printf("%s",cur_mod.file_output);*/
}	

void display_name(void)
{
	if(quiet) return;
	write(writeme,cur_mod.name_type,strlen(cur_mod.name_type));
}	

void display_status(void)
{
	if(quiet) return;
	write(writeme,cur_mod.status,strlen(cur_mod.status));
}	

void display_pausebanner(void)
{
	if(quiet) return;
/*	addstr(pausebanner);*/
}	

void display_extractbanner(void)
{
	if(quiet) return;
/*	addstr(extractbanner);*/
}	

void display_loadbanner(void)
{
	if(quiet) return;
/*	addstr(loadbanner);*/
}	

void display_error(char *myerr, char *myerr_file)
{
	if(quiet) return;
	printf("Non-fatal Error:\n %s: \"%s\".\n",myerr,myerr_file);
	sleep(3);
}	

void display_driver_error(char *myerr)
{
	if(quiet) return;
	printf("Driver error: %s.\n",myerr);
}	

void display_all(void)
{
	if(quiet) return;
	display_version();
	display_driver();
	display_file();
	display_name();
	display_status();
}	

void update_driver_display(void)
{
	if(quiet) return;
	sprintf(cur_mod.driver,
".top1.fra2.fra62.fra98.che5 %s
.top1.fra2.fra62.fra98.che6 %s
.top1.fra2.fra62.fra98.che7 %s
.top1.fra2.fra62.fra77.che1 %s
.top1.fra2.fra62.fra16.lab18 configure -text {%u}\n",
		(md_mode&DMODE_16BITS) ? "deselect":"select",
		(md_mode&DMODE_INTERP) ? "deselect":"select",
		(md_mode&DMODE_STEREO) ? "deselect":"select",
		(dorandom) ? "select":"deselect",
		md_mixfreq);
}

void update_file_display(void)
{
	if(quiet) return;
	sprintf(cur_mod.file_output,"                                                                                                                                                     ");
	sprintf(cur_mod.file_output,"File: %s%s\n",cur_mod.filename,d_text[cur_mod.deleted]); 
}

void update_name_display()
{
	if(quiet) return;

	sprintf(cur_mod.name_type,
".top1.fra2.lab10 configure -text {\"%s\"}
.top1.fra2.fra22.lab25 configure -text {Type: %s}
.top1.fra2.fra22.lab30 configure -text {Periods: %s}\n",
		cur_mod.songname==NULL?"(null)":cur_mod.songname,
		cur_mod.modtype,
		(cur_mod.flags&UF_XMPERIODS) ? "XM type" : "mod type");

}

void update_status_display()
{
	if(quiet) return;

	sprintf(cur_mod.status,
".top1.fra2.fra22.lab27 configure -text {Pattern: %i/ %i}
.top1.fra2.fra22.lab28 configure -text {Position: %i}
.top1.fra2.fra22.lab29 configure -text {Tempo: %i}\n",
		cur_mod.sngpos, cur_mod.numpat,
		cur_mod.patpos, mp_sngspd);
}

void exit_display()
{
	if(quiet) return;
	write(writeme,"destroy .\n",10);
	kill(pid,SIGKILL);
}

void display_instruments()
{
	int t, i, n;
	char string[255];
	unsigned char string2[255];
	unsigned char upasc2lowasc[128];
	FILE *table;

	table=fopen("asc8-to-7.txt","r");
	if(table!=NULL)
	{
		for(i=0;i<128;i++)
		{
			fgets(string,255,table);
			upasc2lowasc[i]=string[4];
		}
	}

	write(writeme,".inst.lb delete 0 end\n",31);

	for(t=0;t<mf->numins;t++)
	{
		if(mf->instruments[t].insname!=NULL)
		{
		/* get rid of {}[]$ chars... You won't belive what a pain in 
			the ass it was to find the bug that makes this code
			essential */
			for(i=0,n=0;i<strlen(mf->instruments[t].insname);i++,n++)
			{
			/* quote all possible characters that the tcl 
			interpreter might try to interpret 
			(yes, there are a lot) */
				if(mf->instruments[t].insname[i]=='{' ||
					mf->instruments[t].insname[i]=='}' ||
					mf->instruments[t].insname[i]=='[' ||
					mf->instruments[t].insname[i]==']' ||
					mf->instruments[t].insname[i]=='$' ||
					mf->instruments[t].insname[i]=='\"' ||
					mf->instruments[t].insname[i]=='\\' ||
					mf->instruments[t].insname[i]=='(' ||
					mf->instruments[t].insname[i]==')' ||
					mf->instruments[t].insname[i]=='@' ||
					mf->instruments[t].insname[i]=='%' ||
					mf->instruments[t].insname[i]=='#')
				{
					string2[n]='\\';
					n++;
				}
				string2[n]=mf->instruments[t].insname[i];
				if(string2[n]>127) 
				{
					string2[n]=upasc2lowasc[string2[n]-128];
				}
			}
			string2[n]=0;
			sprintf(string,".inst.lb insert end \"%s\"\n",string2);
		}
		else sprintf(string,".inst.lb insert end \"\"\n");
		write(writeme,string,strlen(string));
	}
	fclose(table);
}
